@@ -4,13 +4,19 @@ Created on Sun Feb 28 13:08:04 2021
 
 @author: berni
 """
-from phylab.lab import (np, plt, grid, errcor, prnpar, chitest)
-from phylab import lab
+from phylab import (np, plt, grid, errcor, prnpar, chitest)
+import phylab as lab
 
 ''' Variables that control the script '''
 tex = False # LaTeX typesetting maths and descriptions
 gen = False # generate dampened oscillation data points
 
+def naive_tau(signal, time):
+    """ This unfortunately almost never works. Don't use this."""
+    idx = np.argmin(np.abs(signal - np.max(signal)/np.e))
+    tau = time[idx] - time[np.argmax(signal)]
+    return tau
+  
 def est_tau(signal, time, nbins=50):
     """ Attempts to find rough estimate of signal's damping time tau.
     WARNING: This is very much not guaranteed to work, use at your own risk. """
@@ -62,4 +68,4 @@ for time, pos, idx in zip([t_a, t_b], [x_a, x_b], [0, 1]):
     axs[idx].plot(time, ofs + amp*np.exp(-time/tau_f), label=f'$\\tau$-fit = {tau_f:.2f}')
     axs[idx].plot(time[peaks], pos[peaks], 'g.')
     axs[idx].legend(loc ='best')
-plt.show()
\ No newline at end of file
+plt.show()
