@@ -14,9 +14,11 @@ from collections import namedtuple
 # Scientific library imports
 import numpy as np
 from scipy.optimize import curve_fit
+from scipy.optimize import differential_evolution
 from scipy.odr import odrpack
 from scipy import signal as sg
 from scipy import special as sp
+from scipy import stats
 from matplotlib import pyplot as plt
 from matplotlib import cm
 from matplotlib import dates
@@ -24,6 +26,7 @@ from matplotlib import dates
 # Standard argument order for periodical functions:
 # A: Amplitude, frq: frequency, phs: phase, ofs: DC offset, tau: damp time
 args = namedtuple('pars', 'A frq phs ofs')
+goodness = namedtuple('ANOVA', 'chi ndof chi_pval R aR F F_pval')
 
 def sine(t, A, frq, phs=0, ofs=0):
     """ Sinusoidal model function. Standard argument order """
@@ -63,30 +66,30 @@ def coope_circ(coords, Xc=0, Yc=0, Rc=1):
     x, y = coords
     return Xc*x + Yc*y + Rc
 
-def gaussian(x, mean=0, sigma=1):
+def gaussian(x, mean=0, sigma=1, scale=1):
     """ Gaussian or normal probability distribution. """
-    return np.exp( - np.square((x - mean)/(sigma))/2.) / sigma*np.sqrt(2*np.pi)
+    return scale*np.exp( - np.square((x - mean)/(sigma))/2.) / sigma*np.sqrt(2*np.pi)
 
-def gaussian_CDF(x, mean=0, sigma=1):
+def gaussian_CDF(x, mean=0, sigma=1, scale=1):
     """ Gaussian or normal cumulative distribution. """
-    return (1 + sp.erf((x - mean) / (np.sqrt(2)*sigma)))/2.
+    return scale*(1 + sp.erf((x - mean) / (np.sqrt(2)*sigma)))/2.
 
-def lorentzian(x, x0=0, HWHM=1):
+def lorentzian(x, x0=0, HWHM=1, scale=1):
     """ Cauchy - Lorentz - Breit - Wigner probability distribution. """
-    return (HWHM / ((x - x0)**2 + HWHM**2))/np.pi
+    return scale*(HWHM / ((x - x0)**2 + HWHM**2))/np.pi
 
-def lorentzian_CDF(x, x0=0, HWHM=1):
+def lorentzian_CDF(x, x0=0, HWHM=1, scale=1):
     """ Cauchy - Lorentz - Breit - Wigner cumulative distribution. """
-    return np.arctan((x - x0) / HWHM)/np.pi + 0.5
+    return scale*(np.arctan((x - x0) / HWHM)/np.pi + 0.5)
 
-def voigt(x, sigma=1, gamma=1):
+def voigt(x, sigma=1, gamma=1, scale=1):
     """ Voigt profile / probability density function. """
     r2sigma = np.sqrt(2)*sigma
-    return np.real(sp.wofz((x + 1j*gamma)/r2sigma)) / r2sigma*np.sqrt(np.pi)
+    return scale*np.real(sp.wofz((x + 1j*gamma)/r2sigma)) / r2sigma*np.sqrt(np.pi)
 
-def logistic(t, L=1, k=1, t0=0):
+def logistic(x, L=1, k=1, x0=0):
     """ Logistic function or sigmoid curve. """
-    return L/(1 + np.exp(k*(t0 - t)))
+    return L/(1 + np.exp(k*(x0 - x)))
 
 def fder(f, x, pars):
     return np.gradient(f(x, *pars), 1)
@@ -187,9 +190,70 @@ def chisq(model, x, y, alpha, beta, varnames, pars=None, dy=None):
     if dy is None:
         return np.array([[((y - fxmodel(a, b))**2).sum() for a in alpha] for b in beta])
     return np.array([[(((y - fxmodel(a, b))/dy)**2).sum() for a in alpha] for b in beta])
+        
+def residual_squares(pars, model, coords, unc=1):
+    """ Sum of squared errors as a function of pars to be minimized by 
+    differential evolution algorithm. This cannot follow standard
+    argument order, therefore is not meant to be used directly. """
+    x, y = coords
+    exval = model(x, *pars)
+    return np.sum(((y - exval)/unc) ** 2)
 
-def R_squared(observed, predicted):
-    return 1. - (np.var(observed - predicted)/np.var(observed))
+def R_squared(observed, predicted, uncertainty=1):
+    """ Returns R square measure of goodness of fit for predicted model. """
+    weight = 1./uncertainty
+    return 1. - (np.var((observed - predicted)*weight) / np.var(observed*weight))
+
+def adjusted_R(model, coords, popt, unc=1):
+    """ Returns adjusted R square test for optimal parameters popt. """
+    x, y = coords
+    R = R_squared(y, model(x, *popt), uncertainty=unc)
+    adj = (R - 1.) / (1 - len(popt)/len(y))
+    return 1. + adj, R
+
+def Ftest(model, coords, popt, unc=1):
+    """ Fisher test for variance of fitted vs constant model. """
+    x, y = coords
+    SSE = residual_squares(popt, model, coords, unc=unc)
+    chired = SSE / (len(y) - len(popt))
+    SSM = residual_squares(popt, model, [x, np.mean(y)], unc=unc)
+    return SSM/len(popt) / chired
+
+def fit_test(model, coords, popt, unc=1, v=False):
+    """
+    Evaluates goodness of fit metrics for best fit parameters popt with model
+    to data coords as defined in goodness named tuple.
+
+    Parameters
+    ----------
+    model : callable
+        Model function that minimizes square residuals |y - model(x)|^2.
+    coords : array_like
+        List or array of [x, y] measured data coordinates.
+    popt : array_like
+        Optimal values for the parameters of the model.
+    unc : array_like, optional
+        Uncertainties associated to y data coordinates. The default is 1.
+
+    Returns
+    -------
+    named_tuple
+        Results of goodness-of-fit tests.
+
+    """
+    x, y = coords
+    ddof = len(popt)
+    chisq, ndof, resn = chitest(model(x, *popt), y, unc=unc, ddof=ddof)
+    chi_pval = stats.chi2.sf(chisq, ndof)
+    adj_R, R = adjusted_R(model, coords, popt)
+    SSM = residual_squares(popt, model, [x, np.mean(y)], unc=unc)
+    F = SSM/ddof / (chisq/ndof)
+    F_pval = stats.f.sf(F, ddof, ndof)
+    ANOVA = goodness(chisq, ndof, chi_pval, R, adj_R, F, F_pval)
+    if v:
+        print(ANOVA)
+        #prnpar(pars=ANOVA, perr=np.zeros(len(ANOVA)), manual=ANOVA._fields)
+    return ANOVA
 
 def errcor(covm):
     """ Computes parameter error and correlation matrix from covariance. """
@@ -207,12 +271,17 @@ def prncor(corm, model=None, manual=None):
         for j in range(1 + i, np.size(corm, 1)):
             print('corr_%s-%s = %.3f' % (pnms[i], pnms[j], corm[i][j]))
 
-def prnpar(pars, perr, model=None, prec=2, manual=None):
+def prnpar(pars, perr=None, model=None, prec=2, manual=None):
     """ Pretty print fit parameters to specified precision %.<prec>f """
     pnms = getfullargspec(model)[0][1:] if manual is None else manual
-    dec = np.abs((np.log10(perr) - prec)).astype(int) if all(perr) > 0 else np.ones_like(pars)
-    for nam, par, err, d in zip(pnms, pars, perr, dec):
-        print(f'{nam} = {par:.{d}f} +- {err:.{d}f}')
+    if perr is None:
+        perr = np.empty(shape=pars.shape)
+    for nam, par, err in zip(pnms, pars, perr):
+        if err is not None and err > 0:
+            dec = np.abs((np.log10(err) - prec)).astype(int)
+            print(f'{nam} = {par:.{dec}f} +- {err:.{dec}f}')
+        else:
+            print(f'{nam} = {par:.{prec}f}')
 
 def RMS(seq):
     """ Evaluates Root Mean Square of data sequence through Numpy module. """
@@ -330,7 +399,9 @@ def propfit(model, xmes, ymes, dx=0., dy=None, p0=None, alg='lm',
         the same dy passed as argument if propagation was not necessary.
 
     """
-    deff = dy
+    if np.isscalar(dy):
+        dy = np.full(shape=ymes.shape, fill_value=dy)
+    deff = dy if dy is not None else np.ones_like(ymes)
     plist = []; elist = []
     for n in range(max_iter):
         popt, pcov = curve_fit(model, xmes, ymes, p0, deff, method=alg,
@@ -350,7 +421,7 @@ def propfit(model, xmes, ymes, dx=0., dy=None, p0=None, alg='lm',
             prnpar(popt, perr, model)
             # Chi square test
             chisq, ndof = chitest(model(xmes, *popt), ymes, deff, ddof=len(popt))[:-1]
-            print(f'Normalized chi square: {chisq/ndof:.2e}')
+            print(f'reduced chi square: {chisq/ndof:.2e}')
             # Normalized parameter covariance
             print('Correlation matrix:\n', pcor)
             break
@@ -397,6 +468,18 @@ def ODRfit(model, xmes, ymes, dx=0, dy=1, p0=None):
     print('Chi square/ndof = %.1f/%d' % (out.sum_square, len(ymes)-len(p0)))
     return popt, pcov
 
+def gen_init(model, coords, bounds, unc=1):
+    """ Differential evolution algorithm to guess valid initial parameter
+    values within bounds in order to fit model to coords. """
+    if bounds is None:
+        bounds = []
+        for i in len(getfullargspec(model)[0] - 1):
+            bounds.append([-1e8, 1e8])
+
+    result = differential_evolution(residual_squares, bounds,
+                                    args=[model, coords, unc], seed=42)
+    return result.x
+
 def outlier(model, xmes, ymes, dx=None, dy=None, pars=None, thr=5, out=False):
     """ Removes outliers from measured data. A sampled point is considered an
     outlier if it has absolute deviation y - model(x, *pars) > thr*dy. """
@@ -725,7 +808,7 @@ def FFT(time, signal, window=None, beta=0, specres=None):
         return freq, tran, fres, frstd
     return freq, tran
 
-# UTILITIES FOR MANAGING DATA FILES
+# UTILITIES FOR MANAGING DATA AND FILES
 def srange(data, x, x_min=0, x_max=1e9):
     """ Returns sub-array containing data inside selected range over
         dynamic variable x in [x_min, x_max]. If x is equal to data,
@@ -758,6 +841,16 @@ def std_unc(measure, ADC=None):
     unc = np.diff(V)/2/np.sqrt(12)
     return np.append(unc, np.mean(unc))
 
+def synth_data(model, domain, pars=None, wnoise=None, npts=100):
+    """ Creates noisy data arrays of npts points shaped like model(domain). """
+    if len(domain) == 2:
+        domain = np.linspace(start=domain[0], stop=domain[1], num=npts)
+    if wnoise is None:
+        wnoise = [0, 1]
+    ideal = model(domain, *pars) if pars is not None else model(domain) 
+    noise = np.random.normal(loc=wnoise[0], scale=wnoise[1], size=domain.shape)
+    return ideal + noise, domain
+    
 def interleave(a, b):
     """ Join two sequences a & b by alternating their elements. """
     merged = [None]*(len(a)+len(b))
